from jesse.store import store
from jesse import helpers
from jesse.services import logger


def save_daily_portfolio_balance():
    balances = []

    # add exchange balances
    for key, e in store.exchanges.storage.items():
        balances.append(e.assets[helpers.app_currency()])

    # add open position values
    for key, pos in store.positions.storage.items():
        if pos.is_open:
            balances.append(pos.pnl)

    total = sum(balances)
    store.app.daily_balance.append(total)
    logger.info('Saved daily portfolio balance: {}'.format(round(total, 2)))



import jesse.helpers as jh
from jesse.enums import order_types, sides, order_statuses
from jesse.models import Order
from jesse.config import config, reset_config
from jesse.enums import exchanges, timeframes, order_types, order_flags, order_roles
from jesse.routes import router
from jesse.store import store
from jesse.services import selectors
from jesse.models import Position, Exchange

position: Position = None
exchange: Exchange = None


def set_up_without_fee(is_margin_trading=False):
    reset_config()
    config['env']['exchanges'][exchanges.SANDBOX]['type'] = 'margin'
    config['env']['exchanges'][exchanges.SANDBOX]['fee'] = 0
    config['env']['exchanges'][exchanges.SANDBOX]['assets'] = [
        {'asset': 'USDT', 'balance': 1000},
        {'asset': 'BTC', 'balance': 0},
    ]
    if is_margin_trading:
        # used only in margin trading
        config['env']['exchanges'][exchanges.SANDBOX]['type'] = 'margin'
        config['env']['exchanges'][exchanges.SANDBOX]['settlement_currency'] = 'USDT'
    config['app']['trading_mode'] = 'backtest'
    config['app']['considering_exchanges'] = ['Sandbox']
    router.set_routes([(exchanges.SANDBOX, 'BTCUSDT', '5m', 'Test19')])
    store.reset(True)

    global position
    global exchange
    position = selectors.get_position(exchanges.SANDBOX, 'BTCUSDT')
    position.current_price = 50
    exchange = selectors.get_exchange(exchanges.SANDBOX)


def test_cancel_order():
    set_up_without_fee()

    order = Order({
        'id': jh.generate_unique_id(),
        'exchange': exchange.name,
        'symbol': 'BTCUSDT',
        'type': order_types.LIMIT,
        'price': 129.33,
        'qty': 10.2041,
        'side': sides.BUY,
        'status': order_statuses.ACTIVE,
        'created_at': jh.now_to_timestamp(),
    })

    assert order.is_canceled is False

    order.cancel()

    assert order.is_canceled is True
    assert order.canceled_at == jh.now_to_timestamp()


def test_execute_order():
    set_up_without_fee()

    order = Order({
        'id': jh.generate_unique_id(),
        'symbol': 'BTCUSDT',
        'exchange': exchange.name,
        'type': order_types.LIMIT,
        'price': 129.33,
        'qty': 10.2041,
        'side': sides.BUY,
        'status': order_statuses.ACTIVE,
        'created_at': jh.now_to_timestamp(),
    })

    assert order.is_executed is False
    assert order.executed_at is None

    order.execute()

    assert order.is_executed is True
    assert order.executed_at == jh.now_to_timestamp()



from typing import Union

import numpy as np

from .high_pass import high_pass
from .supersmoother import supersmoother


def roofing(candles: np.ndarray, hp_period=48, lp_period=10, source_type="close", sequential=False) -> Union[
    float, np.ndarray]:
    """
    Roofing Filter indicator by John F. Ehlers

    :param candles: np.ndarray
    :param period: int - default=20
    :param source_type: str - default: "close"
    :param sequential: bool - default=False

    :return: float | np.ndarray
    """

    if not sequential and len(candles) > 240:
        candles = candles[-240:]

    hpf = high_pass(candles, period=hp_period, source_type=source_type, sequential=True)

    res = supersmoother(hpf, cutoff=lp_period, sequential=True)

    if sequential:
        return res
    else:
        return None if np.isnan(res[-1]) else res[-1]



import jesse.helpers as jh
import jesse.services.logger as logger
from jesse.exceptions import NegativeBalance
from jesse.models import Order
from jesse.enums import sides, order_types
from jesse.libs import DynamicNumpyArray
import numpy as np
from jesse.services import selectors


class Exchange:
    name = ''
    fee_rate = None

    # current holding assets
    assets = {}
    # used for calculating available balance in margin mode:
    temp_reduced_amount = {}
    # current available assets (dynamically changes based on active orders)
    available_assets = {}
    # used for calculating final performance metrics
    starting_assets = {}

    buy_orders = {}
    sell_orders = {}

    def __init__(self, name: str, starting_assets: list, fee_rate: float, exchange_type: str, settlement_currency: str):
        self.name = name
        self.type = exchange_type.lower()

        for item in starting_assets:
            self.assets[item['asset']] = item['balance']
            self.buy_orders[item['asset']] = DynamicNumpyArray((10, 2))
            self.sell_orders[item['asset']] = DynamicNumpyArray((10, 2))
            self.temp_reduced_amount[item['asset']] = 0

        # margin only: make sure trading routes exist in starting_assets
        if self.type == 'margin':
            from jesse.routes import router
            for r in router.routes:
                base = jh.base_asset(r.symbol)
                if base not in self.assets:
                    self.assets[base] = 0
                if base not in self.buy_orders:
                    self.buy_orders[base] = DynamicNumpyArray((10, 2))
                if base not in self.sell_orders:
                    self.sell_orders[base] = DynamicNumpyArray((10, 2))
                if base not in self.temp_reduced_amount:
                    self.temp_reduced_amount[base] = 0

        self.starting_assets = self.assets.copy()
        self.available_assets = self.assets.copy()
        # in margin mode, start from 0 balance for self.available_assets
        # which acts as a temp variable
        if self.type == 'margin':
            for k in self.available_assets:
                self.available_assets[k] = 0
        self.fee_rate = fee_rate
        self.settlement_currency = settlement_currency.upper()

    def tradable_balance(self, symbol=''):
        if self.type == 'spot':
            if symbol == '':
                raise ValueError
            quote_asset = jh.quote_asset(symbol)
            return self.available_assets[quote_asset]
        else:
            temp_credit = self.assets[self.settlement_currency]
            # we need to consider buy and sell orders of ALL pairs
            # also, consider the value of all open positions
            for asset in self.assets:
                if asset == self.settlement_currency:
                    continue

                position = selectors.get_position(self.name, asset + self.settlement_currency)
                if position is None:
                    continue

                if position.is_open:
                    # add unrealized PNL
                    temp_credit += position.pnl

                # subtract worst scenario orders' used margin
                sum_buy_orders = (self.buy_orders[asset][:][:, 0] * self.buy_orders[asset][:][:, 1]).sum()
                sum_sell_orders = (self.sell_orders[asset][:][:, 0] * self.sell_orders[asset][:][:, 1]).sum()
                if position.is_open:
                    if position.type == 'long':
                        sum_buy_orders += position.value
                    else:
                        sum_sell_orders -= abs(position.value)
                temp_credit -= max(abs(sum_buy_orders), abs(sum_sell_orders))

            return temp_credit

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # MARGIN ONLY
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    def charge_fee(self, amount):
        if self.type == 'spot':
            return

        fee_amount = abs(amount) * self.fee_rate
        new_balance = self.assets[self.settlement_currency] - fee_amount
        logger.info(
            'Charged {} as fee. Balance for {} on {} changed from {} to {}'.format(
                round(fee_amount, 2), self.settlement_currency, self.name,
                round(self.assets[self.settlement_currency], 2),
                round(new_balance, 2),
            )
        )
        self.assets[self.settlement_currency] = new_balance

    def add_realized_pnl(self, realized_pnl: float):
        if self.type == 'spot':
            return

        new_balance = self.assets[self.settlement_currency] + realized_pnl
        logger.info('Added realized PNL of {}. Balance for {} on {} changed from {} to {}'.format(
            round(realized_pnl, 2),
            self.settlement_currency, self.name,
            round(self.assets[self.settlement_currency], 2),
            round(new_balance, 2),
        ))
        self.assets[self.settlement_currency] = new_balance

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # SPOT and a bit of MARGIN
    #
    # in margin, we only modify self.available_assets[base_asset]
    # which is used to detect whether or not the order reduce_only
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    def on_order_submission(self, order: Order, skip_market_order=True):
        base_asset = jh.base_asset(order.symbol)
        quote_asset = jh.quote_asset(order.symbol)

        # skip market order at the time of submission because we don't have
        # the exact order.price. Instead, we call on_order_submission() one
        # more time at time of execution without "skip_market_order=False".
        if order.type == order_types.MARKET and skip_market_order:
            return

        # in margin, we only update available_asset's value which is used for detecting reduce_only orders
        if self.type == 'margin':
            self.available_assets[base_asset] += order.qty
            if order.side == sides.BUY:
                self.buy_orders[base_asset].append(np.array([order.qty, order.price]))
            else:
                self.sell_orders[base_asset].append(np.array([order.qty, order.price]))
            # self.available_assets[quote_asset] -= order.qty * order.price
            return

        # used for logging balance change
        temp_old_quote_available_asset = self.available_assets[quote_asset]
        temp_old_base_available_asset = self.available_assets[base_asset]

        if order.side == sides.BUY:
            quote_balance = self.available_assets[quote_asset]
            self.available_assets[quote_asset] -= (abs(order.qty) * order.price) * (1 + self.fee_rate)
            if self.available_assets[quote_asset] < 0:
                raise NegativeBalance(
                    "Balance cannot go below zero in spot market. Available capital at {} for {} is {} but you're trying to sell {}".format(
                        self.name, quote_asset, quote_balance, abs(order.qty * order.price)
                    )
                )
        # sell order
        else:
            base_balance = self.available_assets[base_asset]
            new_base_balance = base_balance + order.qty
            if new_base_balance < 0:
                raise NegativeBalance(
                    "Balance cannot go below zero in spot market. Available capital at {} for {} is {} but you're trying to sell {}".format(
                        self.name, base_asset, base_balance, abs(order.qty)
                    )
                )

            self.available_assets[base_asset] -= abs(order.qty)

        temp_new_quote_available_asset = self.available_assets[quote_asset]
        if jh.is_debuggable('balance_update') and temp_old_quote_available_asset != temp_new_quote_available_asset:
            logger.info(
                'Available balance for {} on {} changed from {} to {}'.format(
                    quote_asset, self.name,
                    round(temp_old_quote_available_asset, 2), round(temp_new_quote_available_asset, 2)
                )
            )
        temp_new_base_available_asset = self.available_assets[base_asset]
        if jh.is_debuggable('balance_update') and temp_old_base_available_asset != temp_new_base_available_asset:
            logger.info(
                'Available balance for {} on {} changed from {} to {}'.format(
                    base_asset, self.name,
                    round(temp_old_base_available_asset, 2), round(temp_new_base_available_asset, 2)
                )
            )

    def on_order_execution(self, order: Order):
        base_asset = jh.base_asset(order.symbol)
        quote_asset = jh.quote_asset(order.symbol)

        if order.type == order_types.MARKET:
            self.on_order_submission(order, skip_market_order=False)

        if self.type == 'margin':
            if order.side == sides.BUY:
                # find and set order to [0, 0] (same as removing it)
                for index, item in enumerate(self.buy_orders[base_asset]):
                    if item[0] == order.qty and item[1] == order.price:
                        self.buy_orders[base_asset][index] = np.array([0, 0])
                        break
            else:
                # find and set order to [0, 0] (same as removing it)
                for index, item in enumerate(self.sell_orders[base_asset]):
                    if item[0] == order.qty and item[1] == order.price:
                        self.sell_orders[base_asset][index] = np.array([0, 0])
                        break
            return

        # used for logging balance change
        temp_old_quote_asset = self.assets[quote_asset]
        temp_old_quote_available_asset = self.available_assets[quote_asset]
        temp_old_base_asset = self.assets[base_asset]
        temp_old_base_available_asset = self.available_assets[base_asset]

        # works for both buy and sell orders (sell order's qty < 0)
        self.assets[base_asset] += order.qty

        if order.side == sides.BUY:
            self.available_assets[base_asset] += order.qty
            self.assets[quote_asset] -= (abs(order.qty) * order.price) * (1 + self.fee_rate)
        # sell order
        else:
            self.available_assets[quote_asset] += abs(order.qty) * order.price * (1 - self.fee_rate)
            self.assets[quote_asset] += abs(order.qty) * order.price * (1 - self.fee_rate)

        temp_new_quote_asset = self.assets[quote_asset]
        if jh.is_debuggable('balance_update') and temp_old_quote_asset != temp_new_quote_asset:
            logger.info(
                'Balance for {} on {} changed from {} to {}'.format(
                    quote_asset, self.name,
                    round(temp_old_quote_asset, 2), round(temp_new_quote_asset, 2)
                )
            )
        temp_new_quote_available_asset = self.available_assets[quote_asset]
        if jh.is_debuggable('balance_update') and temp_old_quote_available_asset != temp_new_quote_available_asset:
            logger.info(
                'Balance for {} on {} changed from {} to {}'.format(
                    quote_asset, self.name,
                    round(temp_old_quote_available_asset, 2), round(temp_new_quote_available_asset, 2)
                )
            )

        temp_new_base_asset = self.assets[base_asset]
        if jh.is_debuggable('balance_update') and temp_old_base_asset != temp_new_base_asset:
            logger.info(
                'Balance for {} on {} changed from {} to {}'.format(
                    base_asset, self.name,
                    round(temp_old_base_asset, 2), round(temp_new_base_asset, 2)
                )
            )
        temp_new_base_available_asset = self.available_assets[base_asset]
        if jh.is_debuggable('balance_update') and temp_old_base_available_asset != temp_new_base_available_asset:
            logger.info(
                'Balance for {} on {} changed from {} to {}'.format(
                    base_asset, self.name,
                    round(temp_old_base_available_asset, 2), round(temp_new_base_available_asset, 2)
                )
            )

    def on_order_cancellation(self, order: Order):
        base_asset = jh.base_asset(order.symbol)
        quote_asset = jh.quote_asset(order.symbol)

        # in margin, we only update available_asset's value which is used for detecting reduce_only orders
        if self.type == 'margin':
            self.available_assets[base_asset] -= order.qty
            # self.available_assets[quote_asset] += order.qty * order.price
            if order.side == sides.BUY:
                # find and set order to [0, 0] (same as removing it)
                for index, item in enumerate(self.buy_orders[base_asset]):
                    if item[0] == order.qty and item[1] == order.price:
                        self.buy_orders[base_asset][index] = np.array([0, 0])
                        break
            else:
                # find and set order to [0, 0] (same as removing it)
                for index, item in enumerate(self.sell_orders[base_asset]):
                    if item[0] == order.qty and item[1] == order.price:
                        self.sell_orders[base_asset][index] = np.array([0, 0])
                        break
            return

        # used for logging balance change
        temp_old_quote_available_asset = self.available_assets[quote_asset]
        temp_old_base_available_asset = self.available_assets[base_asset]

        if order.side == sides.BUY:
            self.available_assets[quote_asset] += (abs(order.qty) * order.price) * (1 + self.fee_rate)
        # sell order
        else:
            self.available_assets[base_asset] += abs(order.qty)

        temp_new_quote_available_asset = self.available_assets[quote_asset]
        if jh.is_debuggable('balance_update') and temp_old_quote_available_asset != temp_new_quote_available_asset:
            logger.info(
                'Available balance for {} on {} changed from {} to {}'.format(
                    quote_asset, self.name,
                    round(temp_old_quote_available_asset, 2), round(temp_new_quote_available_asset, 2)
                )
            )
        temp_new_base_available_asset = self.available_assets[base_asset]
        if jh.is_debuggable('balance_update') and temp_old_base_available_asset != temp_new_base_available_asset:
            logger.info(
                'Available balance for {} on {} changed from {} to {}'.format(
                    base_asset, self.name,
                    round(temp_old_base_available_asset, 2), round(temp_new_base_available_asset, 2)
                )
            )



import math
from typing import Union

import numpy as np

from .supersmoother import supersmoother


def reflex(candles: np.ndarray, period=20, source_type="close", sequential=False) -> Union[float, np.ndarray]:
    """
    Reflex indicator by John F. Ehlers

    :param candles: np.ndarray
    :param period: int - default=20
    :param source_type: str - default: "close"
    :param sequential: bool - default=False

    :return: float | np.ndarray
    """

    if not sequential and len(candles) > 240:
        candles = candles[-240:]

    ssf = supersmoother(candles, cutoff=period / 2, source_type=source_type, sequential=True)

    rf = np.full_like(ssf, 0)
    ms = np.full_like(ssf, 0)
    sums = np.full_like(ssf, 0)

    for i in range(ssf.shape[0]):
        if not (i < period):
            slope = (ssf[i - period] - ssf[i]) / period
            sum = 0
            for t in range(1, period + 1):
                sum = sum + (ssf[i] + t * slope) - ssf[i - t]
            sum = sum / period
            sums[i] = sum

            ms[i] = 0.04 * sums[i] * sums[i] + 0.96 * ms[i - 1]
            if ms[i] > 0:
                rf[i] = sums[i] / math.sqrt(ms[i])

    if sequential:
        return rf
    else:
        return None if np.isnan(rf[-1]) else rf[-1]



import math
from typing import Union

import numpy as np
from jesse.helpers import get_candle_source



def high_pass(candles: np.ndarray, period=48, source_type="close", sequential=False) -> Union[float, np.ndarray]:
    """
    High Pass Filter indicator by John F. Ehlers

    :param candles: np.ndarray
    :param period: int - default=48
    :param source_type: str - default: "close"
    :param sequential: bool - default=False

    :return: float | np.ndarray
    """

    if not sequential and len(candles) > 240:
        candles = candles[-240:]

    source = get_candle_source(candles, source_type=source_type)

    hpf = np.full_like(source, 0)

    for i in range(source.shape[0]):
        if not (i < 2):
            alpha_arg = 2 * math.pi / (period * 1.414)
            alpha1 = (math.cos(alpha_arg) + math.sin(alpha_arg) - 1) / math.cos(alpha_arg)
            hpf[i] = math.pow(1.0 - alpha1 / 2.0, 2) * (source[i] - 2 * source[i - 1] + source[i - 2]) + 2 * (1 - alpha1) * hpf[i - 1] - math.pow(1 - alpha1, 2) * hpf[i - 2]

    if sequential:
        return hpf
    else:
        return None if np.isnan(hpf[-1]) else hpf[-1]



import talib
from collections import namedtuple

import numpy as np

from jesse.helpers import get_candle_source

DamianiVolatmeter = namedtuple('DamianiVolatmeter', ['vol', 'anti' ])


def damiani_volatmeter(candles: np.ndarray, vis_atr=13, vis_std=20, sed_atr=40, sed_std=100, threshold=1.4, source_type="close", sequential=False) -> DamianiVolatmeter:
    """
    Damiani Volatmeter

    :param candles: np.ndarray
    :param vis_atr: int - default=13
    :param vis_std: int - default=20
    :param sed_atr: int - default=40
    :param sed_std: int - default=100
    :param threshold: float - default=1.4
    :param source_type: str - default: "close"
    :param sequential: bool - default=False

    :return: float | np.ndarray
    """

    if not sequential and len(candles) > 240:
        candles = candles[-240:]

    source = get_candle_source(candles, source_type=source_type)

    lag_s = 0.5

    vol = np.full_like(source, 0)
    t = np.full_like(source, 0)

    atrvis = talib.ATR(candles[:, 3], candles[:, 4], candles[:, 2], timeperiod=vis_atr)
    atrsed = talib.ATR(candles[:, 3], candles[:, 4], candles[:, 2], timeperiod=sed_atr)

    for i in range(source.shape[0]):
        if not (i < sed_std):
            vol[i]=atrvis[i] / atrsed[i] + lag_s * (vol[i - 1] - vol[i - 3])
            anti_thres = np.std(source[i - vis_std:i]) / np.std(source[i - sed_std:i])
            t[i] = threshold - anti_thres

    if sequential:
        return DamianiVolatmeter(vol, t)
    else:
        return DamianiVolatmeter(vol[-1], t[-1])



from jesse.config import config
from jesse.models import Exchange


class ExchangesState:
    def __init__(self):
        self.storage = {}

        for name in config['app']['considering_exchanges']:
            starting_assets = config['env']['exchanges'][name]['assets']
            fee = config['env']['exchanges'][name]['fee']
            exchange_type = config['env']['exchanges'][name]['type']
            settlement_currency = config['env']['exchanges'][name]['settlement_currency']
            self.storage[name] = Exchange(name, starting_assets, fee, exchange_type, settlement_currency)



# import pytest
#
# import jesse.services.selectors as selectors
# from jesse.config import config, reset_config
# from jesse.enums import exchanges
# from jesse.exceptions import NegativeBalance
# from jesse.store import store
#
#
# def set_up():
#     """
#
#     """
#     reset_config()
#     config['app']['considering_exchanges'] = [exchanges.SANDBOX]
#     config['app']['trading_exchanges'] = [exchanges.SANDBOX]
#     config['env']['exchanges'][exchanges.SANDBOX]['starting_balance'] = 2000
#     store.reset()
#
#
# def test_decrease_balance():
#     set_up()
#
#     e = selectors.get_exchange(exchanges.SANDBOX)
#     assert e.balance == 2000
#     e.decrease_balance(None, 100)
#     assert e.balance == 1900
#
#
# def test_increase_balance():
#     set_up()
#
#     e = selectors.get_exchange(exchanges.SANDBOX)
#     assert e.balance == 2000
#     e.increase_balance(None, 100)
#     assert e.balance == 2100
#
#
# def test_negative_balance_validation():
#     with pytest.raises(NegativeBalance):
#         set_up()
#
#         e = selectors.get_exchange(exchanges.SANDBOX)
#         assert e.balance == 2000
#         e.decrease_balance(None, 3000)



config = {
    # these values are related to the user's environment
    'env': {
        'databases': {
            'postgres_host': '127.0.0.1',
            'postgres_name': 'jesse_db',
            'postgres_port': 5432,
            'postgres_username': 'jesse_user',
            'postgres_password': 'password',
        },

        'caching': {
            'driver': 'pickle'
        },

        'logging': {
            'order_submission': True,
            'order_cancellation': True,
            'order_execution': True,
            'position_opened': True,
            'position_increased': True,
            'position_reduced': True,
            'position_closed': True,
            'shorter_period_candles': False,
            'trading_candles': True,
            'balance_update': True,
        },

        'exchanges': {
            'Sandbox': {
                'fee': 0,
                'type': 'spot',
                # used only in margin trading
                'settlement_currency': 'USDT',
                'assets': [
                    {'asset': 'USDT', 'balance': 10000},
                    {'asset': 'BTC', 'balance': 0},
                ],
            },

            # https://www.bitfinex.com
            'Bitfinex': {
                'type': 'margin',
                # used only in margin trading
                'settlement_currency': 'USD',
                'fee': 0.002,
                'assets': [
                    {'asset': 'USDT', 'balance': 10000},
                    {'asset': 'USD', 'balance': 10000},
                    {'asset': 'BTC', 'balance': 0},
                ],
            },

            # https://www.binance.com
            'Binance': {
                'type': 'spot',
                # used only in margin trading
                'settlement_currency': 'USDT',
                'fee': 0.001,
                'assets': [
                    {'asset': 'USDT', 'balance': 10000},
                    {'asset': 'BTC', 'balance': 0},
                ],
            },

            # https://www.binance.com
            'Binance Futures': {
                'type': 'margin',
                # used only in margin trading
                'settlement_currency': 'USDT',
                'fee': 0.0002,
                'assets': [
                    {'asset': 'USDT', 'balance': 10000},
                ],
            },

            # https://testnet.binancefuture.com
            'Testnet Binance Futures': {
                'type': 'margin',
                # used only in margin trading
                'settlement_currency': 'USDT',
                'fee': 0.0002,
                'assets': [
                    {'asset': 'USDT', 'balance': 10000},
                ],
            },

            # https://pro.coinbase.com
            'Coinbase': {
                'type': 'spot',
                # used only in margin trading
                'settlement_currency': 'USDT',
                'fee': 0.005,
                'assets': [
                    {'asset': 'USDT', 'balance': 10000},
                    {'asset': 'BTC', 'balance': 0},
                ],
            },
        },

        # changes the metrics output of the backtest
        'metrics': {
            'sharpe_ratio': True,
            'calmar_ratio': False,
            'sortino_ratio': False,
            'omega_ratio': False,
            'winning_streak': False,
            'losing_streak': False,
            'largest_losing_trade': False,
            'largest_winning_trade': False,
            'total_winning_trades': False,
            'total_losing_trades': False,
        },

        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
        # Optimize mode
        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
        #
        # Below configurations are related to the optimize mode
        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
        'optimization': {
            # sharpe, calmar, sortino, omega
            'ratio': 'sharpe',
        },

        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
        # Data
        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
        #
        # Below configurations are related to the data
        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
        'data': {
            # The minimum number of warmup candles that is loaded before each session.
            'warmup_candles_num': 210,
        }
    },

    # These values are just placeholders used by Jesse at runtime
    'app': {
        # list of currencies to consider
        'considering_symbols': [],
        # The symbol to trade.
        'trading_symbols': [],

        # list of time frames to consider
        'considering_timeframes': [],
        # Which candle type do you intend trade on
        'trading_timeframes': [],

        # list of exchanges to consider
        'considering_exchanges': [],
        # list of exchanges to consider
        'trading_exchanges': [],

        'considering_candles': [],

        # dict of registered live trade drivers
        'live_drivers': {},

        # Accepted values are: 'backtest', 'livetrade', 'fitness'.
        'trading_mode': '',

        # variable used for test-driving the livetrade mode
        'is_test_driving': False,

        # this would enable many console.log()s in the code, which are helpful for debugging.
        'debug_mode': False,
    },
}

backup_config = config.copy()


def set_config(c):
    global config
    config['env'] = c
    # add sandbox because it isn't in the local config file
    config['env']['exchanges']['Sandbox'] = {
        'type': 'spot',
        # used only in margin trading
        'settlement_currency': 'USDT',
        'fee': 0,
        'assets': [
            {'asset': 'USDT', 'balance': 10000},
            {'asset': 'BTC', 'balance': 0},
        ],
    }


def reset_config():
    global config
    config = backup_config.copy()
